//
// "$Id: Fl_Shared_Image.H 8864 2011-07-19 04:49:30Z greg.ercolano $"
//
// Shared image header file for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2010 by Bill Spitzak and others.
//
// This library is free software. Distribution and use rights are outlined in
// the file "COPYING" which should have been included with this file.  If this
// file is missing or damaged, see the license at:
//
//     http://www.fltk.org/COPYING.php
//
// Please report all bugs and problems on the following page:
//
//     http://www.fltk.org/str.php
//

/** \file
   Fl_Shared_Image class. */

#ifndef Fl_Shared_Image_H
#  define Fl_Shared_Image_H

#  include "Fl_Image.H"





// Test function for adding new formats
typedef Fl_Image *(*Fl_Shared_Handler)(const char *name, uchar *header,
                                       int headerlen);

// Shared images class.
/**
  This class supports caching, loading,
  and drawing of image files. Most applications will also want to
  link against the fltk_images library and call the
  fl_register_images()
  function to support standard image formats such as BMP, GIF, JPEG, and PNG.
*/
class FL_EXPORT Fl_Shared_Image : public Fl_Image {

  friend class Fl_JPEG_Image;
  friend class Fl_PNG_Image;
  friend class Fl_Registered_Image;
  void update_from_registered(const Fl_Shared_Image * r, const char * name = 0);

protected:

  static Fl_Shared_Image **images_;	// Shared images
  static int	num_images_;		// Number of shared images
  static int	alloc_images_;		// Allocated shared images
  static Fl_Shared_Handler *handlers_;	// Additional format handlers
  static int	num_handlers_;		// Number of format handlers
  static int	alloc_handlers_;	// Allocated format handlers

  const char	*name_;			// Name of image file
  int		original_;		// Original image?
  int		refcount_;		// Number of times this image has been used
  Fl_Image	*image_;		// The image that is shared
  int		alloc_image_;		// Was the image allocated?

  static int	compare(Fl_Shared_Image **i0, Fl_Shared_Image **i1);

  // Use get() and release() to load/delete images in memory...
  Fl_Shared_Image();
  Fl_Shared_Image(const char *n, Fl_Image *img = 0);
public:
  //Destructor now is puplic.
  virtual ~Fl_Shared_Image();
protected:
  // Deprecated, for mackward compatibility if you use in conslunction eith protected constructor
  // Fl_Shared_Image(const char * name, int W = 0, int H = 0)
  void add();
  // For backward compatibility only.
  void update();

public:
  /** Returns the filename of the shared image */
  // RK: with ABI break this should be const-corrected
  const char	*name() { return name_; }
  /** Returns the number of references of this shared image. When reference is below 1, the image is deleted. */


  // Refcount() is not inline function anymore.
  // Not recompiled applications would return always 1,
  // recompiled will return correct positive value.
  // Also with ABI break this should be const-corrected
  int		refcount();

  // Deprecated but functional. Use operator delete instead.
  void		release();
  // Deprecated
  void		reload();

  // Backward-compatibility, cheap with assure_data_mode(0).
  // Now  copy can be both deleted or release()d. Also sequentioal use of copy()
  // does not cause quality loss because images are always rescalled from original.
  virtual Fl_Image *copy(int W, int H);
  Fl_Image *copy() { return copy(w(), h()); }

  // Cheap with assure_data_mode(0).
  virtual void color_average(Fl_Color c, float i);

  // Cheap with assure_data_mode(0).
  virtual void desaturate();

  virtual void draw(int X, int Y, int W, int H, int cx, int cy);

  // This is not required because it shadows the same in Fl_Image and has the same implementation.
  // However removal would break ABI if somebody would dig a pointer to it (linker error undefined reference to...)
  void draw(int X, int Y) { draw(X, Y, w(), h(), 0, 0); }

  // If assure_data_mode() is 0, does nothing so that either data or
  // drawing cache is always available.
  virtual void uncache();

  static Fl_Shared_Image *find(const char *n, int W = 0, int H = 0);

  // Deprecated but fully functional, use operator new instead.
  // Obtained image can be both release()d or safely deleted.
  static Fl_Shared_Image *get(const char *n, int W = 0, int H = 0);

  static Fl_Shared_Image **images();
  static int		num_images();
  static void		add_handler(Fl_Shared_Handler f);
  static void		remove_handler(Fl_Shared_Handler f);

   ///////////////////  New functions  /////////////////////

  // Basic constructor: In conjunction with operator new it is equivalent to get():
  //   Fl_Shared_Image::get(name, W, H);
  //   new Fl_Shared_Image(name, W, H);
  Fl_Shared_Image(const char * name, int W = 0, int H = 0);

  // Constructor which makes existing image sharable undes particular name.
  // If name is NULL, it assigns unique internall name to it but name() will return 0.
  // Copies of such image will still share the same data as long as they have the same
  // W, H and desaturate() and collor_average() combination results to the same effect.
  // Parameter \a take_over tells Fl_Shared_Image engine if it should destroy image when not
  // needed. In such a case \a image should be created on the heap (using operator new) eg
  //   Fl_Shared_Image(new Fl_RGB_Image(...), 0, 1).
  // It does not say if it will destruct internal data of \a image, this depends how
  // particular \a image was constructed itself. It also deprecates
  //   Fl_JPEG_Image(const char *name, const unsigned char *data)
  //   Fl_PNG_Image(const char * name, const unsigned char * buffer, int	maxsize)
  // although those are fully functional.
  // Note that it might uncache() the \a image (for now it uncaches if it is not
  // Fl_Shared_Image but in the future it might not).
  // If image is shared image, and the name is not null it registers the image
  // under second name. If take_over is 0, it will not release original
  // It he image is Fl_Shared_Image and the name is null, it simply references the image
  // so that it is equivalent with copy consrtuctor: with dereferenced pointer.
  // Note that default value for take_over must be 0 because of people would use shared image
  // as a source it should ressemble copy constructor behaviour...

  Fl_Shared_Image(Fl_Image * image, const char * name = 0, int take_over = 0);


  // Copy constructor. Cheap.
  Fl_Shared_Image(const Fl_Shared_Image &);


  // Assignment operator. Cheap.
  Fl_Shared_Image & operator = (const Fl_Shared_Image &);

  //  Resizes image. Sequential copies or resizes do not cause quality loss because images are resized
  //  from original. If W is 0 it resizes to full-scale original. Cheap with assure_data_mode(0).
  void resize(int W = 0, int H = 0);

  // Sets default policy of newly created shared images should have data available all the time.
  // By switching it off the internal images and data will be created on demand (usually with first draw)
  // and  data could be deleted afterwards, keeping only cache, saving memory and often also CPU resources.
  // By default it is ON (1) for backward compatinility, you might want to switch it off at the beginning
  // of your program. See also assure_data() function.
  static void assure_data_mode(int a);

  // Returns current data policy
  static int assure_data_mode();

  // This switches on  policy for particular image. If you want to have default policy OFF
  // to save memory but you know you need to access data() for particular shared image,
  // you can call assure_data() before accessing data:
  //   image->assure_data();
  //   const char * const * data = image->data();
  // However the data is valid only as long as there was no call to new function fl_draw(Fl_Shared_Image *),
  // because such a call might re-build internal image depending on transformation matrix and graphics state.
  // See also data_w() and data_h().
  void assure_data();

  // If you call fl_draw(Fl_Shared_Image *) function which obeys transdormation matrix, internal image might
  // have different size than encapsulating shell. Following give correct size of data array.
  int data_w() const {return image_? image_->w() : 0;}
  int data_h() const {return image_? image_->h() : 0;}

  // These give dimensions of non-scalled original
  int original_w() const;
  int original_h() const;


};




//
// The following function is provided in the fltk_images library and
// registers all of the "extra" image file formats that are not part
// of the core FLTK library...
//

FL_EXPORT extern void fl_register_images();

#endif // !Fl_Shared_Image_H

//
// End of "$Id: Fl_Shared_Image.H 8864 2011-07-19 04:49:30Z greg.ercolano $"
//
